#include "StdAfx.h"
#include "GGame.h"
#include "GStrings.h"

/*
GGame 
*/
GGame::GGame() : m_pd3d(NULL), m_pd3dd(NULL), m_pspr(NULL), 
	m_activated(true), m_drawskip(false)
{
	m_dev.d3ddt=D3DDEVTYPE_HAL;
	m_dev.buffercount=0;
	m_dev.fullscreen=0;
	m_dev.backformat=D3DFMT_X8R8G8B8;
	m_dev.d3dse=D3DSWAPEFFECT_DISCARD;
	m_dev._size=0;
	m_dev.width=0;
	m_dev.height=0;
	m_dev.stencilformat=D3DFMT_D16;
}

/*
GGame Ҹ
*/
GGame::~GGame()
{
	Release3D();
}

/*
GGame::Release3D()

Ǿ D3Dġ Ѵ.
*/
GRET GGame::Release3D()
{
	SAFE_RELEASE(m_pspr);
	SAFE_RELEASE(m_pd3dd);
	SAFE_RELEASE(m_pd3d);
	return 0;
}

/*
GGame::_getdpp()

m_dev  ġ Ӽ
D3D 밡 D3DPRESENT_PARAMETERS Ѵ.
*/
D3DPRESENT_PARAMETERS GGame::_getdpp()
{
	D3DCAPS9 d3dc;
	m_pd3d->GetDeviceCaps(D3DADAPTER_DEFAULT, m_dev.d3ddt, &d3dc);
	D3DPRESENT_PARAMETERS d3dpp;
	memset(&d3dpp,0,sizeof(d3dpp));
	if(m_dev.fullscreen)
	{
		d3dpp.BackBufferWidth=m_dev.width;
		d3dpp.BackBufferHeight=m_dev.height;
		if(m_dev.backformat==D3DFMT_UNKNOWN)m_dev.backformat=D3DFMT_A8R8G8B8;
	}
	else
	{
		d3dpp.BackBufferWidth=0;
		d3dpp.BackBufferHeight=0;
	}
	d3dpp.Windowed=!m_dev.fullscreen;
	d3dpp.BackBufferCount=m_dev.buffercount;
	d3dpp.BackBufferFormat=m_dev.backformat;
	d3dpp.SwapEffect=m_dev.d3dse;
	if(m_dev.stencilformat!=0)
	{
		d3dpp.AutoDepthStencilFormat=m_dev.stencilformat;
		d3dpp.EnableAutoDepthStencil=true;
	}
	return d3dpp;
}

/*
GGame::Create3DDevice(HWND hWnd)
hWnd ġ   ڵ̴.

IDirect3D9  Ŀ ̸ ̿
IDirect3DDevice9 .
׸ ID3DXSprite ID3DXLine .
*/
GRET GGame::Create3DDevice(HWND hWnd)
{
	m_pd3d=Direct3DCreate9(D3D_SDK_VERSION);
	if(m_pd3d==NULL)return GRET_ERROR_NOD3D;
	D3DPRESENT_PARAMETERS d3dpp=_getdpp();
	if( FAILED(m_pd3d->CreateDevice(D3DADAPTER_DEFAULT, m_dev.d3ddt, hWnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &m_pd3dd)) )return GRET_ERROR_FAIL;
	if(FAILED( D3DXCreateSprite(m_pd3dd, &m_pspr) ))return GRET_ERROR_FAIL;
	m_dev.hwnd=hWnd;
	return 0;
}

/*
GGame::Reset3DDevice()

3Dġ ٽ Ѵ.
*/
GRET GGame::Reset3DDevice()
{
	if(m_pd3d==NULL)return GRET_ERROR_NOD3D;
	D3DPRESENT_PARAMETERS d3dpp=_getdpp();
	if( FAILED(m_pd3dd->Reset(&d3dpp)) )return GRET_ERROR_FAIL;
	return 0;
}

/*
GGame::MessageLoop()

    ޼ ޾Ƶ̰,
 ó Ѵ.
޼  쿡 ӷ Ѵ.
*/
GRET GGame::MessageLoop()
{
	bool islost=false;
	MSG msg;
	if(m_pd3dd)OnCreate();
	while(1)
	{
		if(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			if(msg.message==WM_QUIT)return 0;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			OnProc();
			if(islost)
			{
				if((!m_dev.fullscreen || m_activated) && Reset3DDevice()==0)
				{
					islost=false;
					OnReset();
				}
			}
			else if(m_drawskip)
			{
				OnDrawSkip();
			}
			else
			{
				m_pd3dd->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(64,128,192), 1.f, 0);
				if( SUCCEEDED(m_pd3dd->BeginScene()) )
				{
					m_pspr->Begin(D3DXSPRITE_ALPHABLEND);
					OnDraw();
					m_pspr->End();
					m_pd3dd->EndScene();
					if(m_pd3dd->Present(NULL, NULL, NULL, NULL)==D3DERR_DEVICELOST)
					{
						islost=true;
						OnLost();
					}
				}
			}
			OnPostProc();
			m_drawskip=false;
		}
	}
	return 0;
}

/*
GGame::MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)

ݹԼ WndProc ,  ޼ óѴ.
*/
LRESULT GGame::MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_ACTIVATEAPP:
		m_activated=(wParam!=0);
		break;
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


/*
GGame::SetDrawMode(int mode, int cmode)

׸带 Ѵ.
*/

GRET GGame::SetDrawMode(int mode, int cmode)
{
	if(!m_pd3dd)return GRET_ERROR_NODEVICE;
	m_pspr->Flush(); /*¸   ׵ ׷ȴ ͵
					 Flush()Լ   Ѵ.*/
	m_pd3dd->SetRenderState(D3DRS_COLORWRITEENABLE, cmode);
	switch(mode)
	{
	case DM_Normal: // Result = Src*SrcAlpha + Dest*(1-SrcAlpha)
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		break;
	case DM_NoBlend: // Result = Src*1 + Dest*0
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		break;
	case DM_Add: // Result = Src*SrcAlpha + Dest*1
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		break;
	case DM_Subtract: // Result = Dest*1 - Src*SrcAlpha
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_REVSUBTRACT);
		break;
	case DM_Multiply: // Result = Src*DestColor + Dest*0
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_DESTCOLOR);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		break;
	}
	return 0;
}


/*
GGame::SetDrawSkip(bool skip)

 ӿ ׸⸦   Ѵ.
*/
GRET GGame::SetDrawSkip(bool skip)
{
	m_drawskip=skip;
	return 0;
}

/*
GGame::GetDrawSkip()

 ӿ ׸Ⱑ   Ѵ.
*/
bool GGame::GetDrawSkip()
{
	return m_drawskip;
}

/*
GGame::GetInstance()

 GGame νϽ ȯѴ.
*/
GGame* GGame::GetInstance()
{
	static GGame inst;
	return &inst;
}

/*
GGame::GetDevice()

IDirect3DDevice9 ġ ȯѴ.
*/
LPDIRECT3DDEVICE9 GGame::GetDevice()
{
	return m_pd3dd;
}

/*
GGame::GetSprite()

ID3DXSprite ġ ȯѴ.
*/
LPD3DXSPRITE GGame::GetSprite()
{
	return m_pspr;
}


/*
GGame::OnCreate()

GGame νϽ ޼ 
ȣȴ.
*/
void GGame::OnCreate()
{
	m_sprs.LoadTexture(&DataInFileDirect(L"integral.png"));
	m_sprs.AddSprites(0, 0, 0, 128, 128);
	m_sprs.AddSprites(0, 128, 0, 256, 128);
	m_sprs.AddSprites(0, 128, 128, 256, 192);
}

/*
GGame::OnProc()

GGame   óҶ
ȣȴ.
*/
void GGame::OnProc()
{
}

/*
GGame::OnDraw()

GGame ׸⸦ Ҷ ȣȴ.
ӷƮ   
ȣ   ִ.
*/
void GGame::OnDraw()
{
	SetDrawMode(DM_Normal);
	m_sprs.DrawSprite(0, 10, 10, D3DCOLOR_ARGB(255, 255, 255, 255));
	SetDrawMode(DM_Add);
	m_sprs.DrawSprite(0, 140, 10, D3DCOLOR_ARGB(255, 255, 255, 255));
	SetDrawMode(DM_Subtract);
	m_sprs.DrawSprite(0, 270, 10, D3DCOLOR_ARGB(255, 255, 255, 255));
	SetDrawMode(DM_Multiply);
	m_sprs.DrawSprite(0, 400, 10, D3DCOLOR_ARGB(255, 255, 255, 255));
	SetDrawMode(DM_NoBlend);
	m_sprs.DrawSprite(0, 530, 10, D3DCOLOR_ARGB(255, 255, 255, 255));
}

/*
GGame::OnPostProc()

GGame ׸ó (Ȥ ϰ)
ó    ȣȴ.
*/
void GGame::OnPostProc()
{
}

/*
GGame::OnLost()

GGame D3Dġ Ҿ  ȣȴ.
*/
void GGame::OnLost()
{
	m_pspr->OnLostDevice();
}

/*
GGame::OnReset()

GGame D3Dġ   ȣȴ.
*/
void GGame::OnReset()
{
	m_pspr->OnResetDevice();
}

/*
GGame::OnDrawSkip()

 ӿ ׸Ⱑ Ǿ 
OnDrawſ  Լ ȣȴ.
*/
void GGame::OnDrawSkip()
{
}